/**
 *
 * @author blossier
 */
public class Employe implements java.io.Serializable {
    
    /** Constante representant genre masculin
     */
    public static final boolean _HOMME = false;
    
    /** Constante representant genre feminin
     */
    public static final boolean _FEMME = true;

    private boolean genre; 
    private String nom;
    private Cadre superieur;
    
    private static double salaire_base = 1445.38;//Smic au 1/01/14
    
    /** Constructeur
     * @param nom : nom de l'employe
     * @param genre : genre de l'employe
     */
    public Employe(String nom, boolean genre) {
	this.nom = nom;
	this.genre = genre;
        this.superieur = null;
    }

    /** Constructeur
     * @param nom : nom de l'employe
     * @param genre : genre de l'employe
     * @param superieur : superieur de l'employe
     */
    public Employe(String nom, boolean genre, Cadre superieur) throws Exception{
	this(nom,genre);
	this.superieur = superieur;
	superieur.ajouteSubordonne(this);
    }
    
    /**
     * accesseur pour le nom
     * @return le nom de l'employé
     */
    public String getNom() {
    	return this.nom;
    }
   
   /**
     * accesseur pour le genre
     * @return le genre de l'employé
     */
    public boolean getGenre() {
    	return this.genre;
    }

    /**
     * accesseur pour le superieur
     * @return le superieur de l'employé
     */
    public Cadre getSuperieur() {
    	return this.superieur;
    }
    
     /**
     * Conversion en string pour l'affichage, cette fonction est appelée 
     * automatiquement lors de la conversion en String
     * @return le nom de l'employé  
     */
    public String toString() {
        return this.nom;
    }
    
    /**
     * Teste l'égalité, deux employés sont identiques si leur nom et leur genre 
     * sont identiques
     * @param e : l'employé avec lequel tester
     * @return vrai ou faux
     */
    boolean equals(Employe e) {
    	return ((this.nom.equals(e.nom))&&(this.genre == e.genre));
    }  
    
    /**
     * Revalorisation salaire de base
     * @param s : nouveau salaire de base pour tous les employés
     */
    public static void revalorisation(double s) {
        salaire_base=s;
    }
    
     /**
     * Calcul du salaire
     * @return le salaire de l'employé
     */
    public double getSalaire() {
	return salaire_base;
    }
    
    
    // Questions facultatives:
    
    /**
     * Teste si l'employé a pour  supérieur hiérarchique (direct ou indirect) 
     * l'employé donné en argument
     * @parem e : teste si e est le supérieur
     * @return oui si c'est le casn non sino
     */
    public boolean aPourSuperieur(Employe e) {
	if (superieur == null) return false;
	if (superieur.equals(e)) return true;
	return superieur.aPourSuperieur(e);
    }

    /**
     * Change l'affectation de l'employé auprès du cadre en argument
     * @parem c : affectation auprès du cadre c
     */
    public void changementAffectation(Cadre c)throws Exception{
	if (c.aPourSuperieur(this)) {
	    System.out.println("Changement d'affectation impossible car "
                    +c.getNom()+" a pour supérieur hiérarchique "+this.getNom());
	    return; 
	}
	if (this.superieur != null) this.superieur.enleveSubordonne(this);
	c.ajouteSubordonne(this);
	this.superieur = c; 
    }

    
    /**
     * méthode appelée par descriptionService. 
     * Elle affiche le service  avec 
     * l'indentation mis en argument et retourne le nombre de personnes 
     * du service
     */
    int afficheService(String ind) {
	String tiret="";;
	if (ind!=""){
	    tiret ="- ";
	} 
	System.out.println(ind+tiret+this+ 
                " (Salaire: "+ this.getSalaire()+")"); 
	return 1;
    }
    
    // Méthode utilisée par retireEmploye dans la classe Entreprise
    void retireDeEntreprise(Entreprise ent){
	ent.emp.remove(this.getNom());
	System.out.println(this+" a été retirée de l'entreprise.");
    }
}
