
import java.util.HashMap;
import java.util.Iterator;
import java.util.Collection;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

/**
 * Classe entreprise : aucun héritage
 * @author blossier
 */
public class Entreprise implements java.io.Serializable{
    /** tableau des employés permettant la recherche par le nom,
     * L'ajout et le retrait efficace
     */
    HashMap<String, Employe> emp;
    /**
     * Le patron mis à part pour être affiché en premier et traité différement
     */
    Cadre patron;

    /// Constructeur
    public Entreprise(Cadre c) {
	this.patron = c;
	this.emp = new HashMap<String, Employe>();
    }




    /// ajout d'employé
    public void  ajouteEmploye(Employe e) throws Exception {
	// on vérifie si l'employé est  déjà dans cette entreprise 
        //et dans ce cas on lève une exception
	if (this.emp.containsValue(e.getNom())) throw new Exception(e+ 
                " est déjà dans l'entreprise.");
	this.emp.put(e.getNom(), e);
    }

     /**
     * Constructeur qui lit l'entreprise dans un fichier sauvegardé
     * @param nom_fich : le nom du fichier à lire
     * @throws la fonction lance l'exception FileNotFoundException si elle ne trouve pas le fichier
     */
    public  Entreprise(String nom_fich) throws java.io.IOException {
	try {
	    InputStream is = new FileInputStream(nom_fich);
	    ObjectInputStream ois = new ObjectInputStream(is);

	    Entreprise lu = (Entreprise) ois.readObject();
	    this.patron = lu.patron;
	    this.emp = lu.emp;

	    ois.close();
	    is.close();
	} catch (java.lang.ClassNotFoundException e) {
	    System.err.println("Problème à la lecture du fichier "+nom_fich);
	    e.printStackTrace();
	    System.exit(1);
	}
    }


    /**
     * Sauvegarde l'entreprise
     * @param nom_fich : le nom du fichier
     */
    public void sauvegarde(String nom_fich) {
	try {
	    OutputStream os = new FileOutputStream(nom_fich);
	    ObjectOutputStream oos = new ObjectOutputStream(os);

	    oos.writeObject(this);

	    oos.close();
	    os.close();
	} catch (java.io.IOException e) {
	    System.err.println("Problème à l'écriture du fichier "+nom_fich);
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    /// affiche
    public void affiche() {
	Iterator<Employe> it = this.emp.values().iterator();

	System.out.println(this.patron);
	while(it.hasNext()) {
	    System.out.println(it.next());
	}
    }

    ///Remarque : dans la méthode affiche, on peut remplacer l'utilisation 
    //de l'itérateur par une boucle sur une collection. Voir exemple ci-dessous : 
    public String toString() {
	String result = "Le patron "+this.patron+
                " a pour employés :\n";
	Collection<Employe> liste = emp.values();
	for (Employe e : liste) {
	    result = result+" - "+e+"\n";
	}
	return result;
    }

    public Employe rechercheParNom(String nom) throws Exception {
	// on verifie d'abors que ce n'est pas le patron
	if (this.patron.getNom().equals(nom)) {
	    return this.patron;
	} else {
	    // sinon on le recherche parmis les employes
	    Employe res = this.emp.get(nom);
	    if (res != null) {
		return res;
	    } else {
		throw new Exception("Le nom n'est pas bon");
	    }
	}
    }

    //Questions facultatives :
    public void retireEmploye(String nom) throws Exception {
	try {
	    Employe e = rechercheParNom(nom);
	    e.retireDeEntreprise(this);
	}
	catch (Exception excep) {
	    throw new Exception (nom+
                    " n'est pas ou plus un employé de cette entreprise.");
	}
    }
}
